//**********************************************************************
// STCOM.C
//
// Utility di gestione della porta seriale + Datascope
//
// (c) Sergio Tanzilli - E-mail: tanzo@tin.it
//                       Web   : http://www.ststeam.com/tanzilli
//
// Source per Borland Turbo C++ 1.0
//
// Release:
//
// 1.0 - 20/10/90  Prima stesura.
// 1.1 - 20/04/91  Utilizzata per utility HUB
// 1.3 - 06/06/94  Utilizzato per terminali INN
// 1.4 - 15/06/94  Utilizzato per test rete PicNet
// 1.5 - 26/08/95  Migliorie sul datascope
// 1.6 - 23/08/96  Utilizzato per test di rete PicNet
// 1.7 - 28/07/97  Utilizzato per lo YAPP
// 1.8 - 31/07/97  Primo rilascio su www.picpoint.com
//**********************************************************************

#include "stcom.h"
#include "stdio.h"
#include "dos.h"
#include "alloc.h"
#include "stdlib.h"
#include "mem.h"
#include "string.h"

// Variabili globali di libreria

// Attributi Video VGA per datascope

unsigned char TxAttrib;              // Caratteri trasmessi
unsigned char RxAttrib;              // Caratteri ricevuti
unsigned char CrAttrib;              // Cursore

int StComError=0;                    // Errori di libreria

// Flag di avvenuta inizializione ambiente STCOM

int StComInitFlag=0;

int THRE_Int_Counter=0;
int RLS_Int_Counter=0;
int RDA_Int_Counter=0;
int IMS_Int_Counter=0;
int Int_Counter=0;

int ViewMode=HEX_VIEW;

ST_HCOM hCom[2];

//**********************************************************************
// Inizializzazione ambiente STCOM
//**********************************************************************

void StComInit(void)
{
  if (StComInitFlag==1) return;

  hCom[ST_COM1].InitFlag=0;
  hCom[ST_COM1].TimeOut=5000L;
  hCom[ST_COM1].DataScopeFlag=0;
  hCom[ST_COM1].PntBuffer=0;
  hCom[ST_COM1].MaskIntReq=0x10;
  hCom[ST_COM1].IntCom=0x0C;
  hCom[ST_COM1].RegOffset=0x300;

  hCom[ST_COM2].InitFlag=0;
  hCom[ST_COM2].TimeOut=5000L;
  hCom[ST_COM2].DataScopeFlag=0;
  hCom[ST_COM2].PntBuffer=0;
  hCom[ST_COM2].MaskIntReq=0x08;
  hCom[ST_COM2].IntCom=0x0B;
  hCom[ST_COM2].RegOffset=0x200;

  StComInitFlag=1;
  return;
}

//**********************************************************************
// Apertura porta seriale
//
// Com      : ST_COM1, ST_COM2
// BaudRate : 1 -> 115.000
// DataLen  : 5, 6, 7, 8 bit
// StopBit  : 1, 2
// Parity   : 0=No parity,  1=Even, 2=Odd
//
// Ritorno
//
// Rtc      : 0=Ok, -1=Errore (vedi StComError)
//**********************************************************************

int StOpenCom(int Com, long BaudRate, int DataLen, int StopBit, int Parity)         // 0,1,2
{
  unsigned long Divisor;
  unsigned char lcr;
  unsigned AddRegOff;
  int i;

  StComInit();
  if (Com!=ST_COM1 && Com!=ST_COM2) return(-1);

  lcr=0;
  switch(DataLen)
  {
  case 5:
    lcr|=0x00;
    break;
  case 6:
    lcr|=0x01;
    break;
  case 7:
    lcr|=0x02;
    break;
  case 8:
    lcr|=0x03;
    break;
  default:
    StComError=ST_ERRPARMS;
    return(-1);
  }

  switch(StopBit)
  {
  case 1:
    lcr|=0x00;
    break;
  case 2:
    lcr|=0x04;
    break;
  default:
    StComError=ST_ERRPARMS;
    return(-1);
  }

  switch(Parity)
  {
  case ST_NOPARITY:
    lcr|=0x00;
    break;
  case ST_EVEN:
    lcr|=0x08;
    break;
  case ST_ODD:
    lcr|=0x18;
    break;
  default:
    StComError=ST_ERRPARMS;
    return(-1);
  }

  // Controlla se la porta seriale  gi stata aperta

  if (hCom[Com].InitFlag==1 || BaudRate>115200) {
    StComError=ST_ERRPARMS;
    return(-1);
  }

  Divisor=115200/BaudRate;

  // Alloca il buffer di ricezione

  if ((hCom[Com].RxBufferCom=(unsigned char *)malloc(ST_LENCOMBUFFER))==NULL)
  {
    StComError=ST_ERRMEMORY;
    return (-1);
  }

  // DLAB=1, frame e baud rate

  outp(LCR+hCom[Com].RegOffset,lcr|0x80);
  outp(DLL+hCom[Com].RegOffset,(unsigned char)(Divisor&0xFFL));
  outp(DLM+hCom[Com].RegOffset,(unsigned char)((Divisor>>8)&0xFFL));

  outp(LCR+hCom[Com].RegOffset,lcr); // DLAB=0

  StEnableInt(Com);
  hCom[Com].InitFlag=1;

  StComError=ST_NOERROR;
  return(0);
}

//**********************************************************************
// Chiusura porta seriale
//
// Com      : ST_COM1, ST_COM2
//
// Ritorno
//
// Rtc      : 0=Ok, -1=Errore (vedi StComError)
//**********************************************************************

int StCloseCom(int Com)
{
  if (hCom[Com].InitFlag==0)
  {
    StComError=ST_ERRPARMS;
    return(-1);
  }

  free(hCom[Com].RxBufferCom);
  StDisableInt(Com);
  hCom[Com].InitFlag=0;
  ResetBit(ST_COM2,MCR,RTS);

  StComError=ST_NOERROR;
  return(0);
}

//**********************************************************************
// Ricezione di un carattere da COMx
//
// Com      : ST_COM1, ST_COM2
//
// Ritorno
//
// Rtc      : >=0 = Carattere, -1=Timeout
//**********************************************************************

int StComGetChar(int Com)
{
  int Carattere;
  long iTime=0L;

  for (;;)
  {
    if (StComHit(Com))
    {
      Carattere=*hCom[Com].ReadByte;
      hCom[Com].ReadByte++;
      if (hCom[Com].ReadByte>=hCom[Com].LastByte)
        hCom[Com].ReadByte=hCom[Com].FirstByte;
      return(Carattere);
    }
    iTime++;
    delay(1L);
    if (iTime==hCom[Com].TimeOut) return(-1);
  }
}

//**********************************************************************
// Trasmissione di un carattere su COMx
//
// Com      : ST_COM1, ST_COM2
// Carattere: Carattere da trasmettere
//**********************************************************************

void StComPutChar(int Com, char Carattere)
{
  while (!(inp(LSR+hCom[Com].RegOffset)&THRE));
  outp(TXD+hCom[Com].RegOffset,Carattere);
  StDataScope(Com,DSTX,Carattere);
}

//**********************************************************************
// Controlla se ci sono caratteri nel buffer di ricezione
//
// Com      : ST_COM1, ST_COM2
//
// Ritorno
//
// Rtc      : 0=Buffer vuoto, 1=Uno o pi caratteri disponibili
//**********************************************************************

int StComHit(int Com)
{
  if (hCom[Com].ReadByte!=hCom[Com].WriteByte) return(1);
  else return(0);
}

//**********************************************************************
// Abilita l'interrupt da COMx
//
// Com      : ST_COM1, ST_COM2
//
//**********************************************************************

void StEnableInt(int Com)
{
  // Redireziona l'interrupt di servizio di COMx

  hCom[Com].OldEntryPoint=getvect(hCom[Com].IntCom);

  if (Com==ST_COM1) setvect(hCom[Com].IntCom,StIntCom1);
  else              setvect(hCom[Com].IntCom,StIntCom2);

  outp(hCom[Com].RegOffset+LCR,0x03); // No parity, 1 stop bit, DLAB=0

  // Abilita l'interrupt in ricezione
  outp(hCom[Com].RegOffset+IER,0x01); // Enable interrupt data available

  outp(hCom[Com].RegOffset+MCR,0x0B); // Abilita l'uscita del segnale di interrupt
  outp(PICMSK,inp(PICMSK)&(~hCom[Com].MaskIntReq));

  hCom[Com].WriteByte=hCom[Com].ReadByte=hCom[Com].FirstByte=hCom[Com].RxBufferCom;
  hCom[Com].LastByte=hCom[Com].RxBufferCom+ST_LENCOMBUFFER;
}

//**********************************************************************
// Disabilita l'interrupt da COMx
//**********************************************************************

void StDisableInt(int Com)
{
  outp(hCom[Com].RegOffset+LCR,0x03); // No parity, 1 stop bit, DLAB=0
  outp(hCom[Com].RegOffset+IER,0x00); // Disable interrupt data available
  outp(hCom[Com].RegOffset+MCR,0x0F); // Disabilita l'uscita del segnale di interrupt

  outp(PICMSK,inp(PICMSK)|hCom[Com].MaskIntReq);
  setvect(hCom[Com].IntCom,hCom[Com].OldEntryPoint);
}

//**********************************************************************
// Routine di servizio dell'interrupt in ricezione su ST_COM1
//**********************************************************************

void interrupt far StIntCom1(void)
{
  unsigned IIR_Value;

  IIR_Value=inp(IIR+hCom[ST_COM1].RegOffset);

  for (;;) {

    // Interrupt per Receive Line Status

    if (IIR_Value==0x06) {
      RLS_Int_Counter++;
      break;
    }

    // Interrupt per Received Data Available

    if (IIR_Value==0x04) {
      RDA_Int_Counter++;

      *hCom[ST_COM1].WriteByte=inp(hCom[ST_COM1].RegOffset+RXD);
      StDataScope(ST_COM1,DSRX,*hCom[ST_COM1].WriteByte);
      hCom[ST_COM1].WriteByte++;
      if (hCom[ST_COM1].WriteByte>=hCom[ST_COM1].LastByte)
        hCom[ST_COM1].WriteByte=hCom[ST_COM1].FirstByte;
      break;
    }

    // Interrupt per Transmitter Holding Register Empty

    if (IIR_Value==0x02) {
      THRE_Int_Counter++;
      break;
    }

    // Interrupt per Modem Status

    if (IIR_Value==0x00) {
      IMS_Int_Counter++;
      break;
    }
    break;
  }

  Int_Counter++;
  outp(PICEOI,0x20);
}



//**********************************************************************
// Routine di servizio dell'interrupt in ricezione su ST_COM2
//**********************************************************************

void interrupt far StIntCom2(void)
{
  unsigned IIR_Value;

  IIR_Value=inp(IIR+hCom[ST_COM2].RegOffset);

  for (;;) {

    // Interrupt per Receive Line Status

    if (IIR_Value==0x06) {
      RLS_Int_Counter++;
      break;
    }

    // Interrupt per Received Data Available

    if (IIR_Value==0x04) {
      RDA_Int_Counter++;

      *hCom[ST_COM2].WriteByte=inp(hCom[ST_COM2].RegOffset+RXD);
      StDataScope(ST_COM2,DSRX,*hCom[ST_COM2].WriteByte);
      hCom[ST_COM2].WriteByte++;
      if (hCom[ST_COM2].WriteByte>=hCom[ST_COM2].LastByte)
        hCom[ST_COM2].WriteByte=hCom[ST_COM2].FirstByte;
      break;
    }

    // Interrupt per Transmitter Holding Register Empty

    if (IIR_Value==0x02) {
      THRE_Int_Counter++;
      break;
    }

    // Interrupt per Modem Status

    if (IIR_Value==0x00) {
      IMS_Int_Counter++;
      break;
    }
    break;
  }

  Int_Counter++;
  outp(PICEOI,0x20);
}

int ReadComRegister(int Com,int Register)
{
  return(inp(Register+hCom[Com].RegOffset));
}

//**********************************************************************
// Attivazione datascope
//**********************************************************************

void StDataScopeOn(int Com)
{
  union REGS inregs, outregs;

  int86(0x11,&inregs,&outregs);
  if ( (outregs.x.ax&0x0030)==0x0030 )
  {
    TxAttrib=DSMONOTX;
    RxAttrib=DSMONORX;
    CrAttrib=DSMONOCR;
    hCom[ST_COM1].VideoBuffer=MK_FP(0xB000,0x0C80);
    hCom[ST_COM2].VideoBuffer=MK_FP(0xB000,0x0C80);
  }
  else
  {
    TxAttrib=DSCOLOTX;
    RxAttrib=DSCOLORX;
    CrAttrib=DSCOLOCR;
    hCom[ST_COM1].VideoBuffer=MK_FP(0xB800,0x0C80);
    hCom[ST_COM2].VideoBuffer=MK_FP(0xB800,0x0C80);
  }
  hCom[Com].DataScopeFlag=1;
}

//**********************************************************************
// Disattivazione datascope
//**********************************************************************

void StDataScopeOff(int Com)
{
  hCom[Com].DataScopeFlag=0;;
}

//**********************************************************************
// Cambia la tipologia di visualizzazione del datascope
//**********************************************************************

void StSetDataScopeView(int Mode)
{
  ViewMode=Mode;
}

//**********************************************************************
//  Datascope locale
//  Visualizza un byte in formato dump di memoria
//**********************************************************************

void StDataScope(
  int Com,         // ST_COMx
  int Disp,        // Carattere DSTX, DSRX
  char Carattere)  // Carattere
{
  unsigned char Attrib;

  if (hCom[Com].DataScopeFlag==0) return;

  if (Disp==DSTX)
    Attrib=TxAttrib;
  else
    Attrib=RxAttrib;

  if (ViewMode==ASCII_VIEW) {
    *(hCom[Com].VideoBuffer+hCom[Com].PntBuffer)=Carattere;
    hCom[Com].PntBuffer++;
    *(hCom[Com].VideoBuffer+hCom[Com].PntBuffer)=(*(hCom[Com].VideoBuffer+hCom[Com].PntBuffer)&0x08)|Attrib;
    hCom[Com].PntBuffer++;
  } else {
    *(hCom[Com].VideoBuffer+hCom[Com].PntBuffer)=HiNibble(Carattere);
    hCom[Com].PntBuffer++;
    *(hCom[Com].VideoBuffer+hCom[Com].PntBuffer)=(*(hCom[Com].VideoBuffer+hCom[Com].PntBuffer)&0x08)|Attrib;
    hCom[Com].PntBuffer++;

    *(hCom[Com].VideoBuffer+hCom[Com].PntBuffer)=LoNibble(Carattere);
    hCom[Com].PntBuffer++;
    *(hCom[Com].VideoBuffer+hCom[Com].PntBuffer)=(*(hCom[Com].VideoBuffer+hCom[Com].PntBuffer)&0x08)|Attrib;
    hCom[Com].PntBuffer++;
  }

  if (hCom[Com].PntBuffer==80*4*2) hCom[Com].PntBuffer=0;
  *(hCom[Com].VideoBuffer+hCom[Com].PntBuffer+1)=(*(hCom[Com].VideoBuffer+hCom[Com].PntBuffer+1)&0x08)|CrAttrib;
}

//**********************************************************************
// Cambia il timeout in ricezione
//**********************************************************************

int StSetTimeOut(
  int Com,       // ST_COMx
  long To)       // Timeout (mS)
{
  hCom[Com].TimeOut=To;
  return(0);
}

int SetBit(int Com,unsigned char Register,unsigned char Bit)
{
  outp(Register+hCom[Com].RegOffset,inp(Register+hCom[Com].RegOffset)|Bit);
  return 0;
}

int ResetBit(int Com,unsigned char Register,unsigned char Bit)
{
  outp(Register+hCom[Com].RegOffset,inp(Register+hCom[Com].RegOffset)&(~Bit));
  return 0;
}

//**********************************************************************
// Trasmette un buffer di caratteri
//**********************************************************************

int StSendBuffer(int Com,unsigned TxBufferLen,unsigned char *TxBuffer)
{
  int i;

  for (i=0;i<TxBufferLen;i++)
    StComPutChar(Com, TxBuffer[i]);

  return 0;
}

char HiNibble(int Value)
{
  Value=(Value>>4)&0x0F;
  if (Value>=0 && Value<=9)
    Value+='0';
  else
    Value=Value-0x0A+'A';
  return Value;
}

char LoNibble(int Value)
{
  Value=Value&0x0F;
  if (Value>=0 && Value<=9)
    Value+='0';
  else
    Value=Value-0x0A+'A';
  return Value;
}
