//**********************************************************************
// STCOM.H
//
// Header file per STCOM.H
//
// (c) Sergio Tanzilli - E-mail: tanzo@tin.it
//                       Web   : http://www.ststeam.com/tanzilli
//
// Source per Borland Turbo C++ 1.0
//
// Release:
//
// 1.0 - 20/10/90  Prima stesura.
// 1.1 - 20/04/91  Utilizzata per utility HUB
// 1.3 - 06/06/94  Utilizzato per terminali INN
// 1.4 - 15/06/94  Utilizzato per test rete PicNet
// 1.5 - 26/08/95  Migliorie sul datascope
// 1.6 - 23/08/96  Utilizzato per test di rete PicNet
// 1.7 - 28/07/97  Utilizzato per lo YAPP
// 1.8 - 31/07/97  Primo rilascio su www.picpoint.com
//**********************************************************************

typedef struct
{
  int InitFlag;                              // Flag di porta inizializzata
  long TimeOut;

  int DataScopeFlag;                         // Flag di attivazione datascope
  unsigned char far *VideoBuffer;            // Buffer video per datascope
  unsigned PntBuffer;

  unsigned char *LastByte;                   // Ultimo byte Rx Buffer
  unsigned char *WriteByte;                  // Puntatore di scrittura Rx Buffer
  unsigned char *ReadByte;                   // Puntatore di lettura Rx Buffer
  unsigned char *FirstByte;                  // Primo byte Rx Buffer
  unsigned char *RxBufferCom;                // Puntatore Rx Buffer

  unsigned char MaskIntReq;                  // Maschera di int req
  unsigned char IntCom;                      // Vettore di interrupt
  unsigned RegOffset;                        // Indirizzo iniziale registri
  void (interrupt far *OldEntryPoint)();     // Puntatore vecchio int handler
} ST_HCOM;

// Errori

#define ST_NOERROR    0 // Nessun errore
#define ST_ERRPARMS   1 // Parametri errati
#define ST_ERRMEMORY  2 // Fine memoria

// Indirizzi di I/O

#define PICMSK 0x21
#define PICEOI 0x20

// Registri porta seriale

#define TXD 0xF8
#define RXD 0xF8
#define DLL 0xF8
#define DLM 0xF9
#define IER 0xF9
#define IIR 0xFA
#define LCR 0xFB
#define MCR 0xFC
#define LSR 0xFD
#define MSR 0xFE
#define RES 0xFF

// Bit dei registri della porta seriale

#define CTS  0x10
#define RTS  0x02
#define DR   0x01
#define TSRE 0x40
#define THRE 0x20

// Porte seriali

#define ST_COM1    0
#define ST_COM2    1

// Parametri

#define ST_NOPARITY 0
#define ST_EVEN     1
#define ST_ODD      2

// Limiti

#define ST_LENCOMBUFFER 1024

// Provenienza del carattere da visualizzare nel datascope

#define DSTX 0          // Trasmissione
#define DSRX 1          // Ricezione

// Attributi colore per datascope

#define DSMONOTX 0x07   // Attributi scheda mono
#define DSMONORX 0x70
#define DSMONOCR 0xF0
#define DSCOLOTX 0x20   // Attributi scheda colore
#define DSCOLORX 0x70
#define DSCOLOCR 0x40

// Tipo di visualizzazione datascope

#define ASCII_VIEW       0
#define HEX_VIEW         1

// Macro

#define SetFlag(a,b)    (a|=b)
#define ResetFlag(a,b)  (a&=(~b))
#define IfFlagOn(a,b)   (a&b)

// Dichiarazioni funzioni di libreria

void StComInit(void);
int  StOpenCom(int,long,int,int,int);
int  StCloseCom(int);
int  StComGetChar(int);
void StComPutChar(int,char);
int  StComHit(int);
void StEnableInt(int);
void StDisableInt(int);
void interrupt far StIntCom1(void);
void interrupt far StIntCom2(void);
void StDataScopeOn(int);
void StDataScopeOff(int);
void StDataScope(int,int,char);
void StSetDataScopeView(int Mode);
char HiNibble(int Value);
char LoNibble(int Value);
int  StSetTimeOut(int,long);
int  SetBit(int Com,unsigned char Register,unsigned char Bit);
int  ResetBit(int Com,unsigned char Register,unsigned char Bit);
int  StSendBuffer(int Com,unsigned TxBufferLen,unsigned char *TxBuffer);
int  ReadComRegister(int Com,int Register);
