//**********************************************************************
// YAPP
//
// Dos utility for YAPP! programmer
//
// Author:      Sergio Tanzilli
//              tanzilli@picpoint.com, tanzo@tin.it
//              http://www.picpoint.com
//
// Compiled with Borland Turbo C++ version 1.0
// (needs STCOM.C and STCOM.H header files)
//
// Release
//
// 1.0 July 15  , 1997  - First release on www.picpoint.com
// 1.1 January 8, 1998  - Test commands added
// 1.2 January 9, 1998  - Data and clock line in hi state in run mode
// 1.3 July 16  , 1998  - First issue edition on Fare Elettronica
//**********************************************************************

#include <conio.h>
#include <stdio.h>
#include <io.h>
#include <dos.h>
#include <process.h>
#include <stdlib.h>
#include <alloc.h>
#include <mem.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>

#include "stcom.h"

#define HI      "HI"
#define LO      "LO"
#define ZERO    "0"
#define VCC     "VCC"
#define VPP     "VPP"

#define LOAD_CONFIG    0
#define LOAD_DATA      2
#define LOAD_DATA_DM   3
#define READ_DATA      4
#define READ_DATA_DM   5
#define INC_ADDR       6
#define BEGIN_PROG     8

#define RESET_MODE     64
#define PROGRAM_MODE   65
#define RUN_MODE       66

#define SET_CLOCK_HI    67
#define SET_CLOCK_LO    68
#define SET_DATA_HI     69
#define SET_DATA_LO     70

int ReadHexFile(char *FileName, unsigned int *PicBuffer);
int ReadNextLine(FILE *Stream, char *LineBuffer);
int LookingForAck(int Rtc);

void ExitProg(int Rtc);

int nCom=-1;

void main(int argc,char *argv[])
{
  int KeyValue;
  int Rtc;
  int i;
  int Addr;
  int EnableLog=0;
  int LoByte, HiByte;
  int Key;
  int DataVerify=1;
  int JumpNullInstr=0;

  FILE *log;

  char parameter[20];
  char Filename[50];
  char MCLR_descr[10];
  char DATA_descr[10];
  char CLOCK_descr[10];

  unsigned int PicBuffer[1024];      // Program area buffer
  unsigned int PicConfigBuffer[8];   // Config  area buffer

  PicConfigBuffer[7]=0x19;

  // Prompt

  printf("\nYAPP! - Yet Another Pic Programmer - Jul98 MS/DOS version\n");
  printf("Sergio Tanzilli - tanzo@tin.it\n\n");

  // Check the argument list

  if (argc<3) {
    printf("  Sintax error: yapp <filename> /com[1|2] [/{lp|xt|hs|rc}] [/wdte] [/cp] [/log]\n");
    printf("         where: filename is the file .HEX in IN8HEX format\n");
    printf("                com1, com2 is the serial port to use\n");
    printf("                lp, xt, hw, rc is the oscillator mode\n");
    printf("                wdte - enable watch dog timer\n");
    printf("                cp   - enable code protect\n");
    printf("                nv   - no verify\n");
    printf("                j    - jump 0x3FFF locations\n");
    printf("                log  - enable log error on file yapp.log\n\n");
    printf("  Note: If the filename provided is TEST Yapp starts in test mode\n");
    printf("        (example: yapp test /com1).\n");
    exit(2);
  }

  for (i=2;i<argc;i++) {
    strcpy(parameter,argv[i]);

    if (strncmpi("/com1",parameter,5)==0) {
      nCom=ST_COM1;
    }

    if (strncmpi("/com2",parameter,5)==0) {
      nCom=ST_COM2;
    }

    if (strncmpi("/lp",parameter,3)==0) {
      PicConfigBuffer[7]&=0xFC;
      PicConfigBuffer[7]|=0x00;
    }

    if (strncmpi("/xt",parameter,3)==0) {
      PicConfigBuffer[7]&=0xFC;
      PicConfigBuffer[7]|=0x01;
    }

    if (strncmpi("/hs",parameter,3)==0) {
      PicConfigBuffer[7]&=0xFC;
      PicConfigBuffer[7]|=0x02;
    }

    if (strncmpi("/rc",parameter,3)==0) {
      PicConfigBuffer[7]&=0xFC;
      PicConfigBuffer[7]|=0x03;
    }

    if (strncmpi("/wdte",parameter,5)==0) {
      PicConfigBuffer[7]&=0xF7;
      PicConfigBuffer[7]|=0x04;
    }

    if (strncmpi("/cp",parameter,3)==0) {
      PicConfigBuffer[7]&=0xEF;
      PicConfigBuffer[7]|=0x00;
    }

    if (strncmpi("/nv",parameter,3)==0) {
      DataVerify=0;
    }

    if (strncmpi("/j",parameter,2)==0) {
      JumpNullInstr=1;
    }

    if (strncmpi("/log",parameter,4)==0) {
      EnableLog=1;
      if ((log=fopen("YAPP.LOG","wt"))==NULL) {
        printf("  Log file open error\n");
        exit(2);
      }
    }

  }

  if (nCom==-1) {
    printf("  Please specify com port (/COM1 or /COM2)\n");
    exit(2);
  }

  StOpenCom(nCom,9600,8,1,ST_NOPARITY);
  StSetTimeOut(nCom,500L);

  StComPutChar(nCom,RESET_MODE);
  sprintf(MCLR_descr,ZERO);
  sprintf(DATA_descr,HI);
  sprintf(CLOCK_descr,LO);

  if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

  strcpy(Filename,argv[1]);

  if (strncmpi("test",Filename,4)==0) {

    for (;;) {
      printf("Test mode - Select a function:\n");
      printf("  <p>   Program mode - MCLR=VPP, DATA=LO, CLOCK=LO\n");
      printf("  <r>   Reset mode   - MCLR=0  , DATA=LO, CLOCK=LO\n");
      printf("  <n>   Run mode     - MCLR=VCC, DATA=HI, CLOCK=HI\n");
      printf("  <c>   Change clock line state\n");
      printf("  <d>   Change data line state\n");
      printf("  <Esc> Exit\n\n");
      printf("Current state: Mclr=%s, Data=%s, Clock=%s\n",MCLR_descr,DATA_descr,CLOCK_descr);

      Key=getch();

      if (Key==27) {
        StComPutChar(nCom,RUN_MODE);
        sprintf(MCLR_descr,VCC);
        sprintf(DATA_descr,HI);
        sprintf(CLOCK_descr,HI);
        if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
        ExitProg(0);
      }

      // Program mode

      if (toupper(Key)=='P') {
        StComPutChar(nCom,PROGRAM_MODE);
        sprintf(MCLR_descr,VPP);
        sprintf(DATA_descr,LO);
        sprintf(CLOCK_descr,LO);
        if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
      }

      // Reset mode

      if (toupper(Key)=='R') {
        StComPutChar(nCom,RESET_MODE);
        sprintf(MCLR_descr,ZERO);
        sprintf(DATA_descr,LO);
        sprintf(CLOCK_descr,LO);
        if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
      }

      // Run mode

      if (toupper(Key)=='N') {
        StComPutChar(nCom,RUN_MODE);
        sprintf(MCLR_descr,VCC);
        sprintf(DATA_descr,HI);
        sprintf(CLOCK_descr,HI);
        if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
      }

      // Change clock line state

      if (toupper(Key)=='C') {
        if (strcmpi(CLOCK_descr,HI)==0) {
          StComPutChar(nCom,SET_CLOCK_LO);
          sprintf(CLOCK_descr,LO);
        } else {
          sprintf(CLOCK_descr,HI);
          StComPutChar(nCom,SET_CLOCK_HI);
        }

        if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
      }

      // Change data line state

      if (toupper(Key)=='D') {
        if (strcmpi(DATA_descr,HI)==0) {
          StComPutChar(nCom,SET_DATA_LO);
          sprintf(DATA_descr,LO);
        } else {
          sprintf(DATA_descr,HI);
          StComPutChar(nCom,SET_DATA_HI);
        }

        if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
      }

    }

  }
  
  
  //**********************
  // Read program memory
  //**********************
  
  if (strncmpi("read",Filename,4)==0) {

    printf("Read PIC programm to PIC.OUT\n");

    StComPutChar(nCom,RESET_MODE);
    if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

    StComPutChar(nCom,PROGRAM_MODE);
    if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

    StComPutChar(nCom,LOAD_DATA);
    StComPutChar(nCom,0);
    StComPutChar(nCom,0);
    if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

    for (Addr=0;Addr<1024;Addr++) {

      if (kbhit()) {
        if (getch()==27) {
          printf("\nAbort by user...\n");
          StComPutChar(nCom,RUN_MODE);
          if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
          ExitProg(2);
        }
      }

      if ((Addr%8)==0) {
        printf("\n  %04X  ",Addr);
      }

      delay(1L);

      StComPutChar(nCom,READ_DATA);
      if ((LoByte=StComGetChar(nCom))==-1) {
        printf("  Timeout error. Abort\n");
        ExitProg(2);
      }

      if ((HiByte=StComGetChar(nCom))==-1) {
        printf("  Timeout error. Abort\n");
        ExitProg(2);
      }
      
      PicBuffer[Addr]=((HiByte<<8)&0xFF00)|LoByte;
      
      printf(" %04X  ",PicBuffer[Addr]);
      
      StComPutChar(nCom,INC_ADDR);
      if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
    }

    StComPutChar(nCom,RUN_MODE);
    if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

    ExitProg(0);
  }
  
  //***********************
  // Write program memory
  //***********************
  
  printf("  Input file is --> %s\n",Filename);


  Rtc=ReadHexFile(Filename,PicBuffer);

  if (Rtc==-1) {
    printf("  File [%s] not found\n",Filename);
    ExitProg(2);
  } else if (Rtc==-2) {
    printf("  File format error\n");
    ExitProg(2);
  }


  StComPutChar(nCom,PROGRAM_MODE);
  if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

  // Write program area

  printf("  Start writing program area\n");

  for (Addr=0;Addr<1024;Addr++) {

    if (kbhit()) {
      if (getch()==27) {
        printf("\nAbort by user...\n");
        StComPutChar(nCom,RUN_MODE);
        if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
        ExitProg(2);
      }
    }

    if ((Addr%8)==0) {
      printf("\n  %04X  ",Addr);
    }

    if (JumpNullInstr && PicBuffer[Addr]==0x3FFF) {
      printf(" %04X  ",PicBuffer[Addr]);
      StComPutChar(nCom,INC_ADDR);
      if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
      continue;
    }

    StComPutChar(nCom,LOAD_DATA);
    StComPutChar(nCom,PicBuffer[Addr]&0xFF);
    StComPutChar(nCom,(PicBuffer[Addr]>>8)&0xFF);
    if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

    StComPutChar(nCom,BEGIN_PROG);
    if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

    // Verify data programmed

    if (DataVerify) {
      delay(1L);

      StComPutChar(nCom,READ_DATA);
      if ((LoByte=StComGetChar(nCom))==-1) {
        printf("  Timeout error. Abort\n");
        ExitProg(2);
      }

      if ((HiByte=StComGetChar(nCom))==-1) {
        printf("  Timeout error. Abort\n");
        ExitProg(2);
      }

      if (LoByte!=(PicBuffer[Addr]&0xFF) || HiByte!=((PicBuffer[Addr]>>8)&0xFF)) {
        printf("*%04X  ",PicBuffer[Addr]);

        if (EnableLog==1) {
          fprintf(log,"Addr: %04x  Wrote: %04x Read: %02x%02x\n",Addr,PicBuffer[Addr],HiByte,LoByte);
        }
      } else {
        printf(" %04X  ",PicBuffer[Addr]);
      }
    } else {
      printf(" %04X  ",PicBuffer[Addr]);
    }

    StComPutChar(nCom,INC_ADDR);
    if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);
  }

  // Write config area

  printf("\n\n");
  printf("  Start writing config area\n\n");

  StComPutChar(nCom,LOAD_CONFIG);
  StComPutChar(nCom,PicConfigBuffer[7]&0xFF);
  StComPutChar(nCom,(PicConfigBuffer[7]>>8)&0xFF);
  if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

  for (Addr=0;Addr<7;Addr++) {
    StComPutChar(nCom,INC_ADDR);
    if (LookingForAck(StComGetChar(nCom))==-1) break;
  }

  printf("  %04x   %04X ",7,PicConfigBuffer[7]);


  StComPutChar(nCom,BEGIN_PROG);
  if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

  StComPutChar(nCom,RUN_MODE);
  if (LookingForAck(StComGetChar(nCom))==-1) ExitProg(2);

  ExitProg(0);
}


//**********************************************************************
// Read a INTEL HEX 8 bit format file
//
//  0 = Read succefull
// -1 = File not found
// -2 = File format error
//**********************************************************************

int ReadHexFile(char *FileName, unsigned int *PicBuffer)
{
  FILE *inputStream;

  char LineBuffer[80];
  char Comodo[10];

  int LenRec;
  int Addr;
  int i;
  int Rtc;
  int Valore;

  // Init memory buffer

  for (Addr=0;Addr<1024;Addr++) PicBuffer[Addr]=0x3FFF;

  if ((inputStream=fopen(FileName,"rt"))==NULL) {
    return -1;
  }

  for (;;) {
    Rtc=ReadNextLine(inputStream,LineBuffer);

    if (Rtc==-1) {
      fclose(inputStream);
      return -2;
    }

    if (Rtc==3) continue;
    if (Rtc==2) break;

    if (Rtc==0 || Rtc==1) {

      if (LineBuffer[0]!=':' || strlen(LineBuffer)<11) {
        fclose(inputStream);
        return -2;
      }

      // Legge il segnalatore di ultimo record
      sprintf(Comodo,"0x%2.2s",LineBuffer+7);
      sscanf(Comodo,"%x",&Valore);
      if (Valore==1) break;

      // Legge la lunghezza record
      sprintf(Comodo,"0x%2.2s",LineBuffer+1);
      sscanf(Comodo,"%x",&LenRec);

      // Legge l'indirizzo di partenza
      sprintf(Comodo,"0x%4.4s",LineBuffer+3);
      sscanf(Comodo,"%x",&Addr);

      if (Addr>=0x0400) {
        break;
      }

      for (i=0;i<LenRec;i+=2) {
        sprintf(Comodo,"0x%2.2s",LineBuffer+9+i*2);
        sscanf(Comodo,"%x",&Valore);
        PicBuffer[Addr/2+i/2]=Valore&0x00FF;

        sprintf(Comodo,"0x%2.2s",LineBuffer+9+i*2+2);
        sscanf(Comodo,"%x",&Valore);
        PicBuffer[Addr/2+i/2]|=((Valore<<8)&0xFF00);
      }
    }
  }
  fclose(inputStream);
  return 0;
}

//*****************************************
// Read a line from .HEX input file
//
// Return code:
//
//       0 Readed line
//       1 Last line
//       2 End of file
//       3 Empty line
//      -1 Line too long
//*****************************************

int ReadNextLine(FILE *Stream, char *LineBuffer)
{
  int i=0;
  int Carattere;

  for (;;) {
    Carattere=fgetc(Stream);

    if (Carattere==EOF) {
      if (i>0) {
        LineBuffer[i]=0;
        return 1;
      } else {
        return 2;
      }
    }

    if (Carattere==0x0A) {
      if (i>0) {
        LineBuffer[i]=0;
        return 0;
      } else {
        return 3;
      }
    }

    LineBuffer[i]=Carattere;
    i++;
    if (i>=80) return -1;
  }
}

int LookingForAck(int Rtc) {
  if (Rtc==-1) {
    printf("\n");
    printf("  Timeout error. Abort\n");
    return -1;
  } else if (Rtc!='0') {
    printf("\n");
    printf("  Wrong chars from YAPP. Abort\n");
    return -1;
  }
  return 0;
}

void ExitProg(int Rtc) {
  fcloseall();
  StCloseCom(nCom);
  exit(Rtc);
}
