;**************************************************
; Pic by example
;
; PDM1.ASM
; Primo esempio sul Power Down Mode
;
; (c) 1998, Sergio Tanzilli (tanzo@tin.it)
;           Tiziano Galizia (galti@tin.it)
;
; http://www.picpoint.com
;
; Version Apr98
;**************************************************


        PROCESSOR       16F84
        RADIX           DEC
        INCLUDE         "P16F84.INC"

SWITCH1 EQU     0
SWITCH2 EQU     1
LED1    EQU     2

        ORG     0CH

        ;Contatore a 16 bit utilizzato dalla subroutine delay

Count   RES     2

        ;Reset Vector
        ;Punto di inizio del programma al reset della CPU

        ORG     00H

        ;Salta al corpo principale del programma. Questo jump e' necessario
        ;per evitare tutta la parte di codice per la gestione degli
        ;interrupt.

        goto    Start

        ;**********************************************************************
        ;Interrupt vector
        ;Punto di inizio per tutte le subroutine di gestione degli interrupt
        ;**********************************************************************

        ORG     04H

        ;**********************************************************************
        ; Interrupt handler
        ;**********************************************************************

        bcf     INTCON,INTF     ;Azzera nuovamente il flag INTF per consentire all'interrupt di
                                ;ripetersi

        retfie                  ;Ritorna al programma principale


        ;**********************************************************************
        ; Programma principale
        ;**********************************************************************
Start:
        

        bsf     STATUS,RP0      ;Commuta sul secondo banco dei registri per accedere 
                                ;ai registri TRISA e TRISB

        ;Definizione delle linee di I/O sulla porta A (0=Uscita, 1=Ingresso)
        
        movlw   00011111B       ;Definizione della porta A
        movwf   TRISA

        ;Definizione delle linee di I/O sulla porta B

        bsf     TRISB,SWITCH1   ;Ingresso pulsante 
        bsf     TRISB,SWITCH2   ;Ingresso pulsante 
        bcf     TRISB,LED1      ;Uscita led
        bcf     TRISB,3         ;Linea non utilizzata
        bcf     TRISB,4         ;Linea non utilizzata
        bcf     TRISB,5         ;Linea non utilizzata
        bcf     TRISB,6         ;Linea non utilizzata
        bcf     TRISB,7         ;Linea non utilizzata

        ;Mette a zero il bit INTEDG del registro OPTION per far si che' l'interrupt sul pin 
        ;RB0/INT si verifichi sul fronte di discesa del segnale 
        bcf     OPTION_REG,INTEDG

        ;Commuta sul primo banco dei registri
        bcf     STATUS,RP0

        ;Abilita la gestione degli interrupt
        bsf     INTCON,GIE

        ;Abilita in particolare l'interrupt sul pin RB0/INT
        bsf     INTCON,INTE

        ;Spegne il LED1
        bcf     PORTB,LED1

        ;**********************************************************************
        ; Loop principale
        ;
        ; In questo loop viene fatto lampeggiare il LED1 finche' non viene premuto
        ; il pulsante 2. Una volta premuto il pulsante il PICmicro si mette
        ; in Power Down Mode in attesa di un interrupt dal pin RB0/INT che puo'
        ; essere sollecitato con la pressione del pulsante 2.
        ;**********************************************************************

MainLoop

        btfss   PORTB,SWITCH2   ;Se SWITCH2 e' premuto entra in 
        sleep                   ;Power Down Mode

        call    Delay           ;Ritardo software

        ;Se il led e' gia' acceso lo spegne e viceversa

        btfss   PORTB,LED1      ;Led acceso ?
        goto    TurnOnLed1      ;No, lo accende
        goto    TurnOffLed1     ;Si, lo spegne

        ;Accende LED1

TurnOnLed1
        bsf     PORTB,LED1
        goto    MainLoop        ;Continua a lampeggiare


        ;Spegne LED1

TurnOffLed1
        bcf     PORTB,LED1      ;Spegne LED 1
        goto    MainLoop        ;Continua a lampeggiare


        ;**********************************************************************
        ;Subroutine di ritardo software
        ;**********************************************************************

Delay
        clrf    Count
        clrf    Count+1

DelayLoop

        decfsz  Count,1
        goto    DelayLoop

        decfsz  Count+1,1
        goto    DelayLoop

        return

        END
